/*
  findkeys.c

  Finds valid keys for the RSA scheme

  Part of my implementation of the RSA scheme
*/

#include "findkeys.h"
#include <stdio.h>

/*
  Finds keys for the RSA cryptosystem, stored in e (public) and d (private)
  with length of n
*/

void findKeys (unsigned long int * e, unsigned long int * d, unsigned long int * n)
{
  unsigned long int p = 0, q = 0, phi = 0;

  /* Seed the random number generator and get p, q */
  seedRandom ();

  /* get p, q (length = 8,8 bits so p*q = n is at least 15 bits so n < 2^16 (overflow) */
  /* Weird stuff happens when p == q */
  p = findPrime (128, 255);
  do {
    q = findPrime (128, 255);
  } while (q == p);

  fprintf (stderr, "p = %lu, q = %lu\n", p, q);

  /* n = pq, phi = (p - 1)(q - 1) */
  *n = p*q;
  phi = (p - 1) * (q - 1);

  /* Choose e such that gcd (e, phi) == 1 and choose e < n so we don't overflow */
  do {
    *e = 0;

    do {
      *e = random () % *n;
    }
    while (*e % 2 == 0);
  }
  while (gcd (*e, phi) != 1);

  /* find d, the inverse of e (mod phi) */
  *d = inverse (*e, phi);

  return;
}
