/*
  An implementation of the extended euclidean algorithm
  Finds solutions to diophantine equations of the form:
      ax + by = gcd (a, b)

  Arguments:
      number: The number that we want the inverse of
      mod: The base to find the inverse
  Return Value: The inverse, or zero if there isn't one
*/

#include "inverse.h"

unsigned long int inverse (unsigned long int number, unsigned long int mod)
{
  long int quotient = 0, x = 0, y = 1,
    oldx = 1, oldy = 0, newx = 0, newy = 0;
 
  if (gcd (number, mod) != 1)
    {
      return 0;
    }

  /* Algorithm:
     do {
     quotient <- Divide oldx * number + oldy * mod by x * number + y * mod
     newx <- Subtract x * quotient from oldx
     newy <- Subtract y * quotient from oldy
     oldx <- x
     oldy <- y
     x <- newx
     y <- newy
     } while ( x * number + y * mod != 1)
     
  */
  while ( (x * number) + (y * mod) != 1 )
    {
      quotient = ((oldx * number) + (oldy * mod)) / ((x * number) + (y * mod));

      newx = oldx - (x * quotient);
      newy = oldy - (y * quotient);

      oldx = x;
      oldy = y;

      x = newx;
      y = newy;
    }

  if (x < 0)
    {
      x += mod;
    }

  return x;
}
