/*
  rsa-test.c

  Generates a public key and a private key, then proceeds to encrypt then
  decrypt the data on standard input with the RSA scheme, outputting to standard
  output an identical data stream.

  Part of my implementation of the RSA scheme
*/

#include <stdio.h> // printf
#include <assert.h> // assert
#include "findkeys.h"
#include "exponentiation.h"

int main ()
{
  unsigned long int e = 0, d = 0, n = 0, data = 0, cyphertext = 0, decrypted = 0;

  size_t bytesRead = 0;
  
  FILE *input = NULL;
  FILE *output = NULL;

  /*
  input = fopen ("test.c", "r");
  if (input == NULL)
    {
      perror ("Could not open test.c");
      exit (1);
    }

  output = fopen ("out", "w");
  if (output == NULL)
    {
      perror ("Could not open out");
      exit (2);
    }
  */

  /* We just want to read and write to stdin/stdout right now */
  
  input = stdin;
  output = stdout;
 
  /* Generate some public and private keys */

  findKeys (&e, &d, &n);

  fprintf (stderr, "public key = %lu, private key = %lu, length = %lu\n",
	  e, d, n);

  /* Now we will encrypt and decrypt some data */
  while (( bytesRead = fread (&data, 1, 1, input) ))
    {
      /* We have to pad the input with zeros if a full block was not input */
      /*
      if ( bytesRead != 2 )
	{
	  for (loop = bytesRead; loop < 2; ++ loop)
	    {
	      (&data) [loop] = 0;
	    }
	}
      */

      /* IMPORTANT: data < n for RSA to work! */
      assert (data < n);

      /* cyphertext C = M^e mod n */
      cyphertext = exponentiate (data, e, n);

      /* decrypted data = C^d mod n */
      decrypted = exponentiate (cyphertext, d, n);

      /* Do an "integrity check" */
      if (decrypted != data)
	{
	  fprintf (stderr, "Decrypted data does not match input: M = %lu\n", data);
	}
      
      /* Output the decrypted data */
      fwrite (&decrypted, 1, 1, output);
    }


  fclose (input);
  fclose (output);

  return 0;
}
  
      
      
  

  

